import abc
from ..utils import import_name as import_name
from .base import AbstractEntity as AbstractEntity
from abc import abstractmethod
from typing import Any, Tuple

class BaseFormatter(metaclass=abc.ABCMeta):
    name: str = ...
    mimetypes: Tuple[str, ...] = ...
    @abstractmethod
    def decode(self, value: Any) -> Any: ...
    @abstractmethod
    def encode(self, value: Any) -> Any: ...

class AsIsFormatter(BaseFormatter):
    @staticmethod
    def decode(b: Any): ...
    @staticmethod
    def encode(b: Any): ...

class ChainFormatter(BaseFormatter):
    def __init__(self, formatters: Any) -> None: ...
    def decode(self, b: Any): ...
    def encode(self, b: Any): ...

class Registry(dict):
    @classmethod
    def new(cls): ...
    def new_child(self): ...
    def __call__(self, cls: Any) -> None: ...
    def get(self, name: Any): ...

class StringFormatter(BaseFormatter):
    name: str = ...
    @staticmethod
    def decode(b: Any): ...
    @staticmethod
    def encode(b: Any): ...

class FromStringFormatter(BaseFormatter):
    name: str = ...
    @staticmethod
    def decode(b: Any): ...
    @staticmethod
    def encode(b: Any): ...

class NewLineFormatter(BaseFormatter):
    name: str = ...
    linesep: str = ...
    @staticmethod
    def decode(b: Any): ...
    @classmethod
    def encode(cls, b: Any): ...

class BytesNewLineFormatter(NewLineFormatter):
    name: str = ...
    linesep: Any = ...

class PickleFormatter(BaseFormatter):
    name: str = ...
    def __init__(self) -> None: ...
    def decode(self, b: Any): ...
    def encode(self, b: Any): ...

class JsonFormatter(BaseFormatter):
    name: str = ...
    mimetypes: Any = ...
    converters: Any = ...
    def __init__(self): ...
    def decode(self, b: Any): ...
    def encode(self, b: Any): ...
    @classmethod
    def add_converter(cls, klass: Any, conv: Any, score: int = ...): ...

class YamlFormatter(JsonFormatter):
    name: str = ...
    mimetypes: Any = ...
    def __init__(self): ...
    def encode(self, b: Any): ...

class ZLibFormatter(BaseFormatter, metaclass=abc.ABCMeta):
    name: str = ...
    decode: Any = ...
    encode: Any = ...
    def __init__(self) -> None: ...

class LzmaFormatter(BaseFormatter, metaclass=abc.ABCMeta):
    name: str = ...
    encode: Any = ...
    decode: Any = ...
    def __init__(self): ...

class MsgPackFormatter(BaseFormatter, metaclass=abc.ABCMeta):
    name: str = ...
    decode: Any = ...
    encode: Any = ...
    def __init__(self) -> None: ...

class BsonFormatter(BaseFormatter, metaclass=abc.ABCMeta):
    name: str = ...
    decode: Any = ...
    encode: Any = ...
    def __init__(self) -> None: ...

registry: Any

class FormattedEntity(AbstractEntity):
    registry: Any = ...
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def set_config(self, config: Any) -> None: ...
    def decode(self, b: Any): ...
    def encode(self, b: Any): ...
